#include "StdAfx.h"
#include "DeviceMan.h"
#include "misc.h"
#include "resource.h"
#include <setupapi.h>

#define _MAX_DEVPATH_LEN	4096

DeviceManipulator::DeviceManipulator(void)
	: hTargetWnd( NULL )
	, threadState( FOR_NONE )
	, bAppend( FALSE )
{
}

DeviceManipulator::~DeviceManipulator(void)
{
	Join();
}

DWORD DeviceManipulator::ThreadProc(void)
{
	DWORD dwRet = -1;

	switch( threadState )
	{
	case FOR_ENUM:
		dwRet = EnumProc();
		break;

	case FOR_DEL:
		dwRet = DelProc();
		break;
	}

	return dwRet;
}

DWORD DeviceManipulator::EnumProc(void)
{
	std::vector <DISPLAY_LOG> logAll, logPresent;
	std::vector <DISPLAY_LOG>::iterator IterA, IterP;

	SearchOne( logAll, FALSE );
	SearchOne( logPresent, TRUE );

	if ( bAppend ) {
		for ( IterP = foundDevices.begin(); IterP != foundDevices.end(); IterP++ ) {
			for ( IterA = logAll.begin(); IterA != logAll.end(); IterA++ ) {
				if ( (*IterA).path == (*IterP).path ) {
					logAll.erase( IterA );
					break;
				}
			}
		}
	}
	for ( IterP = logPresent.begin(); IterP != logPresent.end(); IterP++ ) {
		for ( IterA = logAll.begin(); IterA != logAll.end(); IterA++ ) {
			if ( (*IterA).path == (*IterP).path ) {
				logAll.erase( IterA );
				break;
			}
		}
	}

	for ( IterA = logAll.begin(); IterA != logAll.end(); IterA++ ) {
		foundDevices.push_back( (*IterA) );
	}

	threadState = FOR_NONE;

	if ( hTargetWnd ) {
		PostMessage( hTargetWnd, WM_DEVMAN_ENUMEND, 0, 0 );
	}

	return 0;
}

DWORD DeviceManipulator::DelProc(void)
{
	HDEVINFO hDevInfo;
	SP_DEVINFO_DATA devInfo;
	SP_DEVINFO_LIST_DETAIL_DATA devInfoListDetail;
	SP_REMOVEDEVICE_PARAMS rmdParams;

	std::vector <DISPLAY_LOG>::iterator Iter;
	int idx;
	LVITEM li;

	BOOL bIgnoreAll = FALSE;
	BOOL bAborted = FALSE;
	int deleted = 0;
	int skipped = 0;
	int selected = 0;

	UINT errID = 0;
	DWORD dwErr = 0;

	for ( Iter = foundDevices.begin( ), idx = 0; Iter != foundDevices.end( ) ; Iter++, idx++ ) {

		li.iItem = idx;

		if ( !(*Iter).selected ) continue;

		hDevInfo = SetupDiCreateDeviceInfoListEx( NULL, NULL, NULL, NULL );
		if ( hDevInfo == INVALID_HANDLE_VALUE ) {
			dwErr = GetLastError();
			errID = IDS_ERR_SetupDiCreateDeviceInfoListEx;
			goto FAILED;
		}

		if ( !SetupDiOpenDeviceInfo( hDevInfo, (*Iter).path.c_str(), NULL, 0, NULL ) ) {
			dwErr = GetLastError();
			errID = IDS_ERR_SetupDiOpenDeviceInfo;
			goto FAILED;
		}

		devInfoListDetail.cbSize = sizeof( devInfoListDetail );
		if ( !SetupDiGetDeviceInfoListDetail( hDevInfo,&devInfoListDetail ) ) {
			dwErr = GetLastError();
			errID = IDS_ERR_SetupDiGetDeviceInfoListDetail;
			goto FAILED;
		}

		devInfo.cbSize = sizeof(devInfo);
		if ( !SetupDiEnumDeviceInfo( hDevInfo, 0, &devInfo ) ) {
			dwErr = GetLastError();
			errID = IDS_ERR_SetupDiEnumDeviceInfo;
			goto FAILED;
		}

		rmdParams.ClassInstallHeader.cbSize = sizeof(SP_CLASSINSTALL_HEADER);
		rmdParams.ClassInstallHeader.InstallFunction = DIF_REMOVE;
		rmdParams.Scope = DI_REMOVEDEVICE_GLOBAL;
		rmdParams.HwProfile = 0;
		if ( !SetupDiSetClassInstallParams( hDevInfo, &devInfo, &rmdParams.ClassInstallHeader, sizeof(rmdParams) ) ) {
			dwErr = GetLastError();
			errID = IDS_ERR_SetupDiSetClassInstallParams;
			goto FAILED;
		}
		if ( !SetupDiCallClassInstaller( DIF_REMOVE, hDevInfo, &devInfo ) ) {
			dwErr = GetLastError();
			errID = IDS_ERR_SetupDiCallClassInstaller;
			goto FAILED;
		}

		SetupDiDestroyDeviceInfoList( hDevInfo );

		if ( hTargetWnd ) {
			PostMessage( hTargetWnd, WM_DEVMAN_DEVCHG, idx, STATE_REMOVED );
		}
		(*Iter).state = STATE_REMOVED;
		deleted++;
		continue;

FAILED:
		if ( hDevInfo != INVALID_HANDLE_VALUE ) SetupDiDestroyDeviceInfoList( hDevInfo );
		if ( hTargetWnd ) {
			PostMessage( hTargetWnd, WM_DEVMAN_DEVCHG, idx, STATE_FAILED );
		}
		(*Iter).state = STATE_FAILED;
		if ( bIgnoreAll ) continue;
		switch( MessageBoxWithString2( hMainWnd, errID, dwErr ) )
		{
		case IDIGNOREALL:
			bIgnoreAll = TRUE;
		case IDIGNORE:
			skipped++;
			continue;
		}
		bAborted = TRUE;
		break;
	}

	if ( bAborted ) {
		MessageBoxWithString( hMainWnd, IDS_ABORTED, MB_OK );
	}
	else if ( skipped > 0 ) {
		MessageBoxWithString( hMainWnd, IDS_COMPLETEDWITHSKIP, MB_OK );
	}
	else if ( !bOptExit ) {
		if ( deleted > 0 ) {
			MessageBoxWithString( hMainWnd, IDS_COMPLETED, MB_OK );
		}
		else {
			MessageBoxWithString( hMainWnd, IDS_NODELETED, MB_OK );
		}
	}

	if ( foundDevices.size() > 0 ) {
		for ( idx = ( (int)foundDevices.size() - 1 ); idx >= 0; idx-- ) {
			if ( foundDevices[idx].state == STATE_REMOVED ) {
				foundDevices.erase( foundDevices.begin() += idx );
				if ( hTargetWnd ) {
					PostMessage( hTargetWnd, WM_DEVMAN_LISTDEL, idx, 0 );
				}
			}
		}
	}

	threadState = FOR_NONE;

	if ( hTargetWnd ) {
		PostMessage( hTargetWnd, WM_DEVMAN_DELEND, 0, 0 );
	}

	return 0;
}

BOOL DeviceManipulator::Search( ENUM_CLASS c, HWND hWnd, BOOL append )
{
	BOOL bRet = FALSE;

	hTargetWnd = hWnd;
	targetClass = c;

	bAppend = append;

	if ( !bAppend ) {
		foundDevices.clear();
	}

	threadState = FOR_ENUM;
	bRet = Create();

	return bRet;
}

void DeviceManipulator::SearchOne( std::vector <DISPLAY_LOG> &log, BOOL bPresent )
{
	HDEVINFO hDevInfo;
	SP_DEVINFO_DATA DeviceInfoData;
	DISPLAY_LOG entry;
	std::vector <DISPLAY_LOG>::iterator Iter;

	BOOL bCheck;
	std::vector <tstring>::iterator IterStr;

	hDevInfo = SetupDiGetClassDevs( &targetClass.guid,
		0, // Enumerator
		0,
		targetClass.flag | ( bPresent ? DIGCF_PRESENT : 0 )
		);

	if (hDevInfo == INVALID_HANDLE_VALUE)
	{
		// Insert error handling here.
		MessageBoxWithString( hMainWnd, IDS_ERR_SetupDiGetClassDevs1, MB_OK | MB_ICONEXCLAMATION, GetLastError() );
		return;
	}

	// Enumerate through all devices in Set.

	DeviceInfoData.cbSize = sizeof(SP_DEVINFO_DATA);
	for ( DWORD i=0; SetupDiEnumDeviceInfo( hDevInfo, i, &DeviceInfoData ); i++ ) {

		DWORD DataT;
		LPTSTR buffer = NULL;
		DWORD buffersize = 0;

		TCHAR instBuf[_MAX_DEVPATH_LEN];
		BOOL bRet;

		entry.selected = FALSE;
		entry.state = STATE_NONE;

		bRet = SetupDiGetDeviceInstanceId(
				hDevInfo,
				&DeviceInfoData, 
				instBuf,
				_countof(instBuf),
				NULL
				);

		entry.path = instBuf;

		if ( denyName.size() > 0 ) {
			for ( IterStr = denyName.begin(), bCheck = FALSE; IterStr != denyName.end(); IterStr++ ) {
				if ( _tcsnicmp( entry.path.c_str(), (*IterStr).c_str(), (*IterStr).size() ) == 0 ) {
					bCheck = TRUE;
					break;
				}
			}
			if ( bCheck ) continue;
		}

		if ( allowName.size() > 0 ) {
			for ( IterStr = allowName.begin(), bCheck = FALSE; IterStr != allowName.end(); IterStr++ ) {
				if ( _tcsnicmp( entry.path.c_str(), (*IterStr).c_str(), (*IterStr).size() ) == 0 ) {
					bCheck = TRUE;
					break;
				}
			}
			if ( !bCheck ) continue;
		}

		//
		// Call function with null to begin with, 
		// then use the returned buffer size (doubled)
		// to Alloc the buffer. Keep calling until
		// success or an unknown failure.
		//
		//  Double the returned buffersize to correct
		//  for underlying legacy CM functions that 
		//  return an incorrect buffersize value on 
		//  DBCS/MBCS systems.
		// 
		while ( !SetupDiGetDeviceRegistryProperty(
				hDevInfo,
				&DeviceInfoData,
				SPDRP_FRIENDLYNAME,
				&DataT,
				(PBYTE)buffer,
				buffersize,
				&buffersize
				) )
		{
			if ( GetLastError() == 	ERROR_INSUFFICIENT_BUFFER )
			{
				// Change the buffer size.
				if ( buffer ) LocalFree( buffer );
				// Double the size to avoid problems on
				// W2k MBCS systems per KB 888609.
				buffer = (LPTSTR)LocalAlloc( LPTR, buffersize * 2 );
			}
			else
			{
				// Insert error handling here.
				break;
			}
		}

		if ( buffer == NULL ) {
			while (!SetupDiGetDeviceRegistryProperty(
				hDevInfo,
				&DeviceInfoData,
				SPDRP_DEVICEDESC,
				&DataT,
				(PBYTE)buffer,
				buffersize,
				&buffersize))
			{
				if ( GetLastError() == ERROR_INSUFFICIENT_BUFFER )
				{
					// Change the buffer size.
					if ( buffer ) LocalFree( buffer );
					// Double the size to avoid problems on
					// W2k MBCS systems per KB 888609.
					buffer = (LPTSTR)LocalAlloc( LPTR, buffersize * 2 );
				}
				else
				{
					// Insert error handling here.
					break;
				}
			}
		}
		if ( buffer == NULL ) {
			TCHAR szTemp[256];
			LoadString( hInst, IDS_GETNAMEFAILED, szTemp, 256 );
			entry.name = szTemp;
		}
		else {
			entry.name = buffer;
			LocalFree( buffer );
		}

		log.push_back( entry );
	}

	//  Cleanup
	SetupDiDestroyDeviceInfoList(hDevInfo);
}

BOOL DeviceManipulator::Delete( HWND hWnd )
{
	hTargetWnd = hWnd;
	BOOL bRet = FALSE;

	if ( foundDevices.size() == 0 ) {
		if ( bOptWarn ) {
			MessageBoxWithString( hWnd, IDS_WARNNODEVICE, MB_OK );
		}
	}
	else {
		threadState = FOR_DEL;
		bRet = Create();
	}

	return bRet;
}

BOOL DeviceManipulator::ApplySortedList( std::vector <tstring> newPathList )
{
	size_t i, j;
	BOOL bRet = FALSE;
	std::vector <DISPLAY_LOG> newDeviceList;

	do {
		for ( i = 0; i < newPathList.size(); i++ ) {
			for ( j = 0; j < foundDevices.size(); j++ ) {
				if ( foundDevices[j].path == newPathList[i] ) {
					newDeviceList.push_back( foundDevices[j] );
					break;
				}
			}
		}
	} while( FALSE );

	bRet = ( foundDevices.size() == newDeviceList.size() ) ? TRUE : FALSE;
	if ( bRet ) {
		foundDevices.swap( newDeviceList );
	}

	return bRet;
}
