#include "stdafx.h"
#include <windows.h>
#include "misc.h"
#include "resource.h"

typedef BOOL ( WINAPI* LPFN_ISWOW64PROCESS )( HANDLE hProcess, PBOOL Wow64Process );
LPFN_ISWOW64PROCESS lpfnIsWow64Process;

typedef void ( WINAPI* LPFN_GETNATIVESYSTEMINFO )( LPSYSTEM_INFO lpSystemInfo );
LPFN_GETNATIVESYSTEMINFO lpfnGetNativeSystemInfo;


SIMPLE_OSVERSION	GetSimpleOSVersion( void )
{
	OSVERSIONINFOEX ver;
	static SIMPLE_OSVERSION soVer = OS_UNKNOWN;

	if ( soVer != OS_UNKNOWN ) return soVer;

	ver.dwOSVersionInfoSize = sizeof(ver);
	GetVersionEx( (LPOSVERSIONINFO)&ver );

	switch( ver.dwPlatformId )
	{
	case VER_PLATFORM_WIN32s:
		soVer = OS_OLD;
		break;

	case VER_PLATFORM_WIN32_WINDOWS:
		soVer = OS_9X;
		break;

	case VER_PLATFORM_WIN32_NT:
		switch( ver.dwMajorVersion )
		{
		case 3:
			soVer = OS_OLD;
			break;

		case 4:
			soVer = OS_NT4;
			break;

		case 5:
			switch( ver.dwMinorVersion )
			{
			case 0:
				soVer = OS_2000;
				break;

			case 1:
				soVer = OS_XP;
				break;

			case 2:
				switch( ver.wProductType )
				{
				case VER_NT_WORKSTATION:
					soVer = OS_XP;
					break;

				default:
					soVer = OS_S2003;
					break;
				}
				break;

			default:
				soVer = OS_NEWER;
				break;
			}
			break;

		case 6:
			switch( ver.dwMinorVersion )
			{
			case 0:
				switch( ver.wProductType )
				{
				case VER_NT_WORKSTATION:
					soVer = OS_VISTA;
					break;

				default:
					soVer = OS_S2008;
					break;
				}
				break;

			case 1:
				switch( ver.wProductType )
				{
				case VER_NT_WORKSTATION:
					soVer = OS_7;
					break;

				default:
					soVer = OS_S2008R2;
					break;
				}
				break;

			default:
				soVer = OS_NEWER;
				break;
			}
			break;

		default:
			soVer = OS_NEWER;
			break;
		}
		break;
	}

	return soVer;
}

int MessageBoxWithString(HWND hWnd, UINT uID, UINT nType, DWORD dwErr)
{
	TCHAR tcError[10], *temp = NULL;
	int nRet, n = 0;

	do {
		if ( temp ) {
			delete[] temp;
		}
		n += 1024;
		temp = new TCHAR[n + 10];
		nRet = LoadString( hInst, uID, temp, n );
	} while( ( nRet + 1 ) == n );

	if ( dwErr != 0 ) {
		_stprintf_s( tcError, _T(":%x"), dwErr );
		_tcscat_s( temp, n + 10, tcError );
	}

	nRet = MessageBox( hWnd, temp, szTitle, nType );
	delete[] temp;

	return nRet;
}

int MessageBoxWithString2(HWND hWnd, UINT uID, DWORD dwErr)
{
	TCHAR tcMessage[256], tcError[10];

	LoadString( hInst, uID, tcMessage, 256 );

	if ( dwErr != 0 ) {
		_stprintf_s( tcError, 10, _T(":%x"), dwErr );
		_tcscat_s( tcMessage, 256, tcError );
	}

	CMessageBox mb( hWnd, tcMessage, szTitle );
	MessageBeep( MB_ICONEXCLAMATION );
	return mb.Show();
}

BOOL MakeAnotherProcessName( TCHAR *path, int len )
{
	TCHAR exePath[_MAX_PATH], drive[_MAX_DRIVE], dir[_MAX_DIR], fname[_MAX_FNAME], ext[_MAX_EXT];

	if ( Isx86() ) {
		path[0] = 0;
		return FALSE;
	}

	GetModuleFileName( hInst, exePath, _countof(exePath) );
	_tsplitpath_s( exePath, drive, _countof(drive), dir, _countof(dir), fname, _countof(fname), ext, _countof(ext) );

	size_t lenfname = _tcslen( fname );
	if ( ( lenfname > 2 ) && ( _tcscmp( fname + lenfname - 2, _T("64") ) == 0 ) ) {
		fname[ lenfname - 2 ] = 0;
	}
	else {
		_tcscat_s( fname, _MAX_FNAME, _T("64") );
	}

	_tmakepath_s( path, len, drive, dir, fname, ext );

	return TRUE;
}

BOOL LaunchApp( TCHAR *path, TCHAR *param, BOOL noui )
{
	SHELLEXECUTEINFO sei = {0};

	sei.cbSize = sizeof(SHELLEXECUTEINFO);
	sei.nShow = SW_NORMAL;
	sei.lpFile = path;
	sei.fMask = noui ? SEE_MASK_FLAG_NO_UI : 0;
	sei.lpParameters = param;

	return ShellExecuteEx( &sei );
}

BOOL IsWow64()
{
	BOOL bRet = FALSE;

	lpfnIsWow64Process = (LPFN_ISWOW64PROCESS)GetProcAddress( GetModuleHandle( _T("kernel32.dll") ), "IsWow64Process" );

	do {
		if ( !lpfnIsWow64Process ) break;

		lpfnIsWow64Process( GetCurrentProcess(), &bRet );

	} while( FALSE );

	return bRet;
}

BOOL Isx86()
{
	BOOL bRet = TRUE;
	SYSTEM_INFO si;

	lpfnGetNativeSystemInfo = (LPFN_GETNATIVESYSTEMINFO)GetProcAddress( GetModuleHandle( _T("kernel32.dll") ), "GetNativeSystemInfo" );

	do {
		if ( !lpfnGetNativeSystemInfo ) break;

		lpfnGetNativeSystemInfo( &si );
		if ( si.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_INTEL ) break;

		bRet = FALSE;

	} while( FALSE );

	return bRet;
}
